# 23 "asllib/Lexer.mll"
 

open Tokens

open Error

module type CONFIG = sig
    (** Allow variables starting with a double underscore (__) *)
    val allow_double_underscore : bool
    val allow_unknown : bool
end

let reserved_keywords = [
    "SAMPLE";
    "UNSTABLE";
    "_"; "any";
    "assume"; "assumes";
    "call"; "cast";
    "class"; "dict";
    "endcase"; "endcatch"; "endclass";
    "endevent"; "endfor"; "endfunc"; "endgetter";
    "endif"; "endmodule"; "endnamespace"; "endpackage";
    "endproperty"; "endrule"; "endsetter"; "endtemplate";
    "endtry"; "endwhile";
    "event"; "export";
    "extends"; "extern"; "feature";
    "gives";
    "iff"; "implies"; "import";
    "intersect"; "intrinsic";
    "invariant"; "list";
    "map"; "module"; "namespace"; "newevent";
    "newmap"; "original";
    "package"; "parallel";
    "port"; "private";
    "profile"; "property"; "protected"; "public";
    "requires"; "rethrow"; "rule";
    "shared"; "signal";
    "template";
    "typeof"; "union";
    "using";
    "ztype";
]

let is_reserved_keyword: string -> bool =
  let tbl: (string, unit) Hashtbl.t = Hashtbl.create (List.length reserved_keywords) in
  let () = List.iter (fun s -> Hashtbl.add tbl s ()) reserved_keywords in
  fun s -> Hashtbl.mem tbl s

(* Get a Tokens.token from the string name of the token.
   A useful utility when working with cmly files.

   Note that this set only contains simple enumerative types, not compound types

   This function's exhaustivity is guaranteed by the test in tests/Lexer. *)
let token_of_string =
 let s t = Some t in
 function
 | "ACCESSOR"           -> s ACCESSOR
 | "AND"                -> s AND
 | "ARRAY"              -> s ARRAY
 | "ARROW"              -> s ARROW
 | "AS"                 -> s AS
 | "ASSERT"             -> s ASSERT
 | "BAND"               -> s BAND
 | "BEGIN"              -> s BEGIN
 | "BEQ"                -> s BEQ
 | "BIARROW"            -> s BIARROW
 | "BIT"                -> s BIT
 | "BITS"               -> s BITS
 | "BNOT"               -> s BNOT
 | "BOOLEAN"            -> s BOOLEAN
 | "BOR"                -> s BOR
 | "CASE"               -> s CASE
 | "CATCH"              -> s CATCH
 | "COLON"              -> s COLON
 | "COLON_COLON"        -> s COLON_COLON
 | "COMMA"              -> s COMMA
 | "CONFIG"             -> s CONFIG
 | "CONSTANT"           -> s CONSTANT
 | "DEBUG"              -> s DEBUG
 | "DIV"                -> s DIV
 | "DIVRM"              -> s DIVRM
 | "DO"                 -> s DO
 | "DOT"                -> s DOT
 | "DOWNTO"             -> s DOWNTO
 | "ELSE"               -> s ELSE
 | "ELSIF"              -> s ELSIF
 | "END"                -> s END
 | "ENUMERATION"        -> s ENUMERATION
 | "EOF"                -> s EOF
 | "EOR"                -> s EOR
 | "EQ"                 -> s EQ
 | "EQ_OP"              -> s EQ_OP
 | "EXCEPTION"          -> s EXCEPTION
 | "FOR"                -> s FOR
 | "FUNC"               -> s FUNC
 | "GEQ"                -> s GEQ
 | "GETTER"             -> s GETTER
 | "GT"                 -> s GT
 | "IF"                 -> s IF
 | "IMPL"               -> s IMPL
 | "IN"                 -> s IN
 | "INTEGER"            -> s INTEGER
 | "LBRACE"             -> s LBRACE
 | "LBRACKET"           -> s LBRACKET
 | "LEQ"                -> s LEQ
 | "LET"                -> s LET
 | "LOOPLIMIT"          -> s LOOPLIMIT
 | "LPAR"               -> s LPAR
 | "LT"                 -> s LT
 | "MINUS"              -> s MINUS
 | "MOD"                -> s MOD
 | "MUL"                -> s MUL
 | "NEQ"                -> s NEQ
 | "NOT"                -> s NOT
 | "OF"                 -> s OF
 | "OR"                 -> s OR
 | "OTHERWISE"          -> s OTHERWISE
 | "PASS"               -> s PASS
 | "PLUS"               -> s PLUS
 | "PLUS_COLON"         -> s PLUS_COLON
 | "POW"                -> s POW
 | "PRAGMA"             -> s PRAGMA
 | "PRINTLN"            -> s PRINTLN
 | "PRINT"              -> s PRINT
 | "RBRACE"             -> s RBRACE
 | "RBRACKET"           -> s RBRACKET
 | "RDIV"               -> s RDIV
 | "REAL"               -> s REAL
 | "RECORD"             -> s RECORD
 | "RECURSELIMIT"       -> s RECURSELIMIT
 | "REPEAT"             -> s REPEAT
 | "RETURN"             -> s RETURN
 | "RPAR"               -> s RPAR
 | "STAR_COLON"         -> s STAR_COLON
 | "SEMI_COLON"         -> s SEMI_COLON
 | "SETTER"             -> s SETTER
 | "SHL"                -> s SHL
 | "SHR"                -> s SHR
 | "SLICING"            -> s SLICING
 | "STRING"             -> s STRING
 | "SUBTYPES"           -> s SUBTYPES
 | "THEN"               -> s THEN
 | "THROW"              -> s THROW
 | "TO"                 -> s TO
 | "TRY"                -> s TRY
 | "TYPE"               -> s TYPE
 | "ARBITRARY"          -> s ARBITRARY
 | "UNREACHABLE"        -> s UNREACHABLE
 | "UNTIL"              -> s UNTIL
 | "VAR"                -> s VAR
 | "WHEN"               -> s WHEN
 | "WHERE"              -> s WHERE
 | "WHILE"              -> s WHILE
 | "WITH"               -> s WITH
 | "LLBRACKET"          -> s LLBRACKET
 | "RRBRACKET"          -> s RRBRACKET
 | _ -> None

(** Convert a lexical token to the symbol it lexes *)
let token_to_symbol = function
  | BNOT               -> "!"
  | COMMA              -> ","
  | LT                 -> "<"
  | SHR                -> ">>"
  | BAND               -> "&&"
  | IMPL               -> "-->"
  | SHL                -> "<<"
  | RBRACKET           -> "]"
  | RRBRACKET          -> "]]"
  | RPAR               -> ")"
  | SLICING            -> ".."
  | EQ                 -> "="
  | LBRACE             -> "{"
  | NEQ                -> "!="
  | MINUS              -> "-"
  | BEQ                -> "<->"
  | LBRACKET           -> "["
  | LLBRACKET          -> "[["
  | LPAR               -> "("
  | DOT                -> "."
  | LEQ                -> "<="
  | POW                -> "^"
  | MUL                -> "*"
  | RDIV               -> "/"
  | EQ_OP              -> "=="
  | BOR                -> "||"
  | PLUS               -> "+"
  | COLON              -> ":"
  | ARROW              -> "=>"
  | BIARROW            -> "<=>"
  | RBRACE             -> "}"
  | COLON_COLON        -> "::"
  | GT                 -> ">"
  | PLUS_COLON         -> "+:"
  | STAR_COLON         -> "*:"
  | SEMI_COLON         -> ";"
  | GEQ                -> ">="
  (* Keywords *)
  | ACCESSOR           -> "accessor"
  | AND                -> "AND"
  | ARRAY              -> "array"
  | AS                 -> "as"
  | ASSERT             -> "assert"
  | BEGIN              -> "begin"
  | BIT                -> "bit"
  | BITS               -> "bits"
  | BOOLEAN            -> "boolean"
  | CASE               -> "case"
  | CATCH              -> "catch"
  | CONFIG             -> "config"
  | CONSTANT           -> "constant"
  | DIV                -> "DIV"
  | DIVRM              -> "DIVRM"
  | DO                 -> "do"
  | DOWNTO             -> "downto"
  | ELSE               -> "else"
  | ELSIF              -> "elsif"
  | END                -> "end"
  | ENUMERATION        -> "enumeration"
  | EOR                -> "XOR"
  | EXCEPTION          -> "exception"
  | FOR                -> "for"
  | FUNC               -> "func"
  | GETTER             -> "getter"
  | IF                 -> "if"
  | IN                 -> "IN"
  | INTEGER            -> "integer"
  | LET                -> "let"
  | LOOPLIMIT          -> "looplimit"
  | MOD                -> "MOD"
  | NOT                -> "NOT"
  | OF                 -> "of"
  | OR                 -> "OR"
  | OTHERWISE          -> "otherwise"
  | PASS               -> "pass"
  | PRAGMA             -> "pragma"
  | PRINTLN            -> "println"
  | PRINT              -> "print"
  | REAL               -> "real"
  | RECORD             -> "record"
  | RECURSELIMIT       -> "recurselimit"
  | REPEAT             -> "repeat"
  | RETURN             -> "return"
  | SETTER             -> "setter"
  | STRING             -> "string"
  | SUBTYPES           -> "subtypes"
  | THEN               -> "then"
  | THROW              -> "throw"
  | TO                 -> "to"
  | TRY                -> "try"
  | TYPE               -> "type"
  | ARBITRARY          -> "ARBITRARY"
  | UNREACHABLE        -> "Unreachable"
  | UNTIL              -> "until"
  | VAR                -> "var"
  | WHEN               -> "when"
  | WHERE              -> "where"
  | WHILE              -> "while"
  | WITH               -> "with"
  | BOOL_LIT _
  | INT_LIT _
  | REAL_LIT _
  | STRING_LIT _
  | BITVECTOR_LIT _
  | MASK_LIT _
  | IDENTIFIER _
  | EOF
  | DEBUG -> assert false (* Complex tokens *)


module Make (Config : CONFIG) = struct

exception LexerError

let new_line lexbuf = Lexing.new_line lexbuf; lexbuf
let bitvector_lit lxm = BITVECTOR_LIT (Bitvector.of_string lxm)
let mask_lit lxm = MASK_LIT (Bitvector.mask_of_string lxm)
let reserved_err s = Error.fatal_unknown_pos @@ (Error.ReservedIdentifier s)

let fatal lexbuf desc =
  AST.
    {
      desc;
      version = V1;
      pos_start = Lexing.lexeme_start_p lexbuf;
      pos_end = Lexing.lexeme_end_p lexbuf;
    }
  |> Error.fatal

let tr_name s = match s with
| "accessor"      -> ACCESSOR
| "AND"           -> AND
| "array"         -> ARRAY
| "as"            -> AS
| "assert"        -> ASSERT
| "begin"         -> BEGIN
| "bit"           -> BIT
| "bits"          -> BITS
| "boolean"       -> BOOLEAN
| "case"          -> CASE
| "catch"         -> CATCH
| "config"        -> CONFIG
| "constant"      -> CONSTANT
| "__debug__"
| "__DEBUG__"     -> DEBUG
| "DIV"           -> DIV
| "DIVRM"         -> DIVRM
| "do"            -> DO
| "downto"        -> DOWNTO
| "else"          -> ELSE
| "elsif"         -> ELSIF
| "end"           -> END
| "enumeration"   -> ENUMERATION
| "XOR"           -> EOR
| "exception"     -> EXCEPTION
| "FALSE"         -> BOOL_LIT false
| "for"           -> FOR
| "func"          -> FUNC
| "getter"        -> GETTER
| "if"            -> IF
| "IN"            -> IN
| "integer"       -> INTEGER
| "let"           -> LET
| "looplimit"     -> LOOPLIMIT
| "MOD"           -> MOD
| "NOT"           -> NOT
| "of"            -> OF
| "OR"            -> OR
| "otherwise"     -> OTHERWISE
| "pass"          -> PASS
| "pragma"        -> PRAGMA
| "println"       -> PRINTLN
| "print"         -> PRINT
| "real"          -> REAL
| "record"        -> RECORD
| "recurselimit"  -> RECURSELIMIT
| "repeat"        -> REPEAT
| "return"        -> RETURN
| "setter"        -> SETTER
| "string"        -> STRING
| "subtypes"      -> SUBTYPES
| "then"          -> THEN
| "throw"         -> THROW
| "to"            -> TO
| "try"           -> TRY
| "TRUE"          -> BOOL_LIT true
| "type"          -> TYPE
| "UNKNOWN"       ->
    if Config.allow_unknown then ARBITRARY
    else fatal_unknown_pos (Error.ObsoleteSyntax s)
| "ARBITRARY"     -> ARBITRARY
| "Unreachable"   -> UNREACHABLE
| "until"         -> UNTIL
| "var"           -> VAR
| "when"          -> WHEN
| "where"         -> WHERE
| "while"         -> WHILE
| "with"          -> WITH
(* Reserved identifiers *)
| x when is_reserved_keyword x -> reserved_err x
| x when not Config.allow_double_underscore
         && ASTUtils.string_starts_with ~prefix:"__" x -> reserved_err x
(* End of reserved identifiers *)
| x               -> IDENTIFIER x

# 369 "asllib/Lexer.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base =
   "\000\000\251\255\252\255\253\255\254\255\255\255\001\000\002\000\
    \253\255\254\255\255\255\003\000\004\000\253\255\000\000\255\255\
    \086\000\205\255\196\000\000\000\209\255\215\255\000\000\001\000\
    \025\000\002\000\224\255\227\255\000\000\233\255\000\000\002\000\
    \236\255\004\000\004\000\000\000\002\000\005\000\244\255\006\000\
    \031\001\248\255\022\000\039\001\009\000\007\000\255\255\252\255\
    \031\000\032\000\055\001\112\001\088\001\152\001\247\255\246\255\
    \232\255\007\000\028\000\239\255\230\255\216\255\208\255\242\255\
    \241\255\036\000\240\255\237\255\235\255\217\255\221\255\228\255\
    \210\255\220\255\211\255\214\255\213\255\000\000\001\000\001\000\
    \001\000\000\000\005\000\001\000\005\000\207\255";
  Lexing.lex_backtrk =
   "\255\255\255\255\255\255\255\255\255\255\255\255\004\000\003\000\
    \255\255\255\255\255\255\004\000\003\000\255\255\001\000\255\255\
    \051\000\255\255\049\000\255\255\255\255\255\255\037\000\036\000\
    \255\255\032\000\255\255\255\255\026\000\255\255\021\000\029\000\
    \255\255\017\000\024\000\255\255\043\000\012\000\255\255\010\000\
    \255\255\255\255\004\000\004\000\033\000\001\000\255\255\255\255\
    \002\000\255\255\255\255\005\000\006\000\255\255\255\255\255\255\
    \255\255\030\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_default =
   "\001\000\000\000\000\000\000\000\000\000\000\000\007\000\007\000\
    \000\000\000\000\000\000\012\000\012\000\000\000\255\255\000\000\
    \255\255\000\000\255\255\255\255\000\000\000\000\255\255\255\255\
    \255\255\255\255\000\000\000\000\255\255\000\000\255\255\255\255\
    \000\000\255\255\255\255\255\255\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\255\255\255\255\255\255\000\000\000\000\
    \048\000\255\255\255\255\255\255\255\255\255\255\000\000\000\000\
    \000\000\255\255\255\255\000\000\000\000\000\000\000\000\000\000\
    \000\000\255\255\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\000\000";
  Lexing.lex_trans =
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\008\000\255\255\013\000\255\255\000\000\
    \045\000\000\000\000\000\000\000\045\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\003\000\010\000\255\255\000\000\064\000\045\000\
    \000\000\255\255\000\000\000\000\075\000\014\000\255\255\015\000\
    \068\000\065\000\058\000\047\000\000\000\000\000\000\000\000\000\
    \048\000\000\000\076\000\074\000\072\000\070\000\069\000\062\000\
    \063\000\059\000\057\000\056\000\049\000\061\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\052\000\060\000\071\000\002\000\009\000\255\255\045\000\
    \046\000\067\000\066\000\045\000\000\000\000\000\000\000\000\000\
    \000\000\082\000\084\000\000\000\077\000\081\000\005\000\078\000\
    \079\000\080\000\083\000\000\000\004\000\042\000\045\000\039\000\
    \041\000\085\000\000\000\000\000\035\000\040\000\027\000\032\000\
    \025\000\023\000\038\000\034\000\031\000\044\000\043\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \022\000\020\000\037\000\030\000\036\000\073\000\019\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\028\000\000\000\033\000\026\000\018\000\000\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\029\000\024\000\021\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\000\000\000\000\
    \255\255\255\255\255\255\255\255\255\255\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\255\255\
    \000\000\000\000\000\000\018\000\000\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\040\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\054\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\040\000\
    \040\000\000\000\000\000\000\000\000\000\049\000\017\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \042\000\000\000\000\000\000\000\000\000\000\000\000\000\051\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\042\000\000\000\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\052\000\000\000\000\000\000\000\000\000\000\000\053\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\000\000\050\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\051\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\051\000\051\000\051\000\051\000\051\000\051\000\052\000\
    \053\000\000\000\000\000\000\000\000\000\000\000\000\000\055\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \053\000\053\000\000\000\000\000\000\000\000\000\000\000\051\000\
    \000\000\051\000\051\000\051\000\051\000\051\000\051\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \053\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000";
  Lexing.lex_check =
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\006\000\007\000\011\000\012\000\255\255\
    \045\000\255\255\255\255\255\255\045\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\006\000\007\000\255\255\035\000\045\000\
    \255\255\048\000\255\255\255\255\023\000\011\000\012\000\014\000\
    \031\000\034\000\037\000\044\000\255\255\255\255\255\255\255\255\
    \044\000\255\255\022\000\023\000\025\000\030\000\030\000\036\000\
    \036\000\037\000\037\000\039\000\042\000\057\000\042\000\042\000\
    \042\000\042\000\042\000\042\000\042\000\042\000\042\000\042\000\
    \049\000\049\000\049\000\049\000\049\000\049\000\049\000\049\000\
    \049\000\049\000\058\000\028\000\000\000\006\000\007\000\016\000\
    \016\000\033\000\065\000\016\000\255\255\255\255\255\255\255\255\
    \255\255\081\000\083\000\255\255\019\000\080\000\000\000\077\000\
    \078\000\079\000\082\000\255\255\000\000\042\000\016\000\016\000\
    \016\000\084\000\255\255\255\255\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\024\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\255\255\016\000\016\000\016\000\255\255\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\016\000\016\000\016\000\016\000\
    \016\000\016\000\016\000\016\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\255\255\255\255\
    \000\000\006\000\007\000\011\000\012\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\048\000\
    \255\255\255\255\255\255\018\000\255\255\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\018\000\
    \018\000\018\000\018\000\018\000\018\000\018\000\018\000\040\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\040\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\040\000\
    \040\000\255\255\255\255\255\255\255\255\043\000\016\000\043\000\
    \043\000\043\000\043\000\043\000\043\000\043\000\043\000\043\000\
    \043\000\255\255\255\255\255\255\255\255\255\255\255\255\050\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\050\000\050\000\
    \050\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \050\000\050\000\050\000\050\000\050\000\050\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\043\000\255\255\
    \052\000\052\000\052\000\052\000\052\000\052\000\052\000\052\000\
    \052\000\052\000\255\255\255\255\255\255\255\255\255\255\040\000\
    \050\000\050\000\050\000\050\000\050\000\050\000\255\255\043\000\
    \051\000\051\000\051\000\051\000\051\000\051\000\051\000\051\000\
    \051\000\051\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\051\000\051\000\051\000\051\000\051\000\051\000\052\000\
    \053\000\255\255\255\255\255\255\255\255\255\255\255\255\053\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \053\000\053\000\255\255\255\255\255\255\255\255\255\255\051\000\
    \255\255\051\000\051\000\051\000\051\000\051\000\051\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \053\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255";
  Lexing.lex_base_code =
   "";
  Lexing.lex_backtrk_code =
   "";
  Lexing.lex_default_code =
   "";
  Lexing.lex_trans_code =
   "";
  Lexing.lex_check_code =
   "";
  Lexing.lex_code =
   "";
}

let rec escaped_string_chars acc lexbuf =
   __ocaml_lex_escaped_string_chars_rec acc lexbuf 0
and __ocaml_lex_escaped_string_chars_rec acc lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 422 "asllib/Lexer.mll"
         ( Buffer.add_char acc '\n'; string_lit acc lexbuf )
# 598 "asllib/Lexer.ml"

  | 1 ->
# 423 "asllib/Lexer.mll"
         ( Buffer.add_char acc '\t'; string_lit acc lexbuf )
# 603 "asllib/Lexer.ml"

  | 2 ->
# 424 "asllib/Lexer.mll"
         ( Buffer.add_char acc '"'; string_lit acc lexbuf )
# 608 "asllib/Lexer.ml"

  | 3 ->
# 425 "asllib/Lexer.mll"
         ( Buffer.add_char acc '\\'; string_lit acc lexbuf )
# 613 "asllib/Lexer.ml"

  | 4 ->
# 426 "asllib/Lexer.mll"
                         ( raise LexerError )
# 618 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_escaped_string_chars_rec acc lexbuf __ocaml_lex_state

and string_lit acc lexbuf =
   __ocaml_lex_string_lit_rec acc lexbuf 6
and __ocaml_lex_string_lit_rec acc lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 429 "asllib/Lexer.mll"
          ( STRING_LIT (Buffer.contents acc) )
# 630 "asllib/Lexer.ml"

  | 1 ->
# 430 "asllib/Lexer.mll"
          ( escaped_string_chars acc lexbuf )
# 635 "asllib/Lexer.ml"

  | 2 ->
# 431 "asllib/Lexer.mll"
          ( Buffer.add_char acc '\n'; new_line lexbuf |> string_lit acc )
# 640 "asllib/Lexer.ml"

  | 3 ->
let
# 432 "asllib/Lexer.mll"
                          lxm
# 646 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 432 "asllib/Lexer.mll"
                              ( Buffer.add_string acc lxm; string_lit acc lexbuf )
# 650 "asllib/Lexer.ml"

  | 4 ->
# 433 "asllib/Lexer.mll"
          ( raise LexerError )
# 655 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_string_lit_rec acc lexbuf __ocaml_lex_state

and c_comments lexbuf =
   __ocaml_lex_c_comments_rec lexbuf 11
and __ocaml_lex_c_comments_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 441 "asllib/Lexer.mll"
                  ( token      lexbuf )
# 667 "asllib/Lexer.ml"

  | 1 ->
# 442 "asllib/Lexer.mll"
                  ( c_comments lexbuf )
# 672 "asllib/Lexer.ml"

  | 2 ->
# 443 "asllib/Lexer.mll"
                  ( new_line lexbuf |> c_comments )
# 677 "asllib/Lexer.ml"

  | 3 ->
# 444 "asllib/Lexer.mll"
                  ( c_comments lexbuf )
# 682 "asllib/Lexer.ml"

  | 4 ->
# 445 "asllib/Lexer.mll"
                  ( raise LexerError  )
# 687 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_c_comments_rec lexbuf __ocaml_lex_state

and token lexbuf =
   __ocaml_lex_token_rec lexbuf 16
and __ocaml_lex_token_rec lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 453 "asllib/Lexer.mll"
                               ( new_line lexbuf |> token         )
# 699 "asllib/Lexer.ml"

  | 1 ->
# 454 "asllib/Lexer.mll"
                               ( token lexbuf                     )
# 704 "asllib/Lexer.ml"

  | 2 ->
# 455 "asllib/Lexer.mll"
                               ( token lexbuf                     )
# 709 "asllib/Lexer.ml"

  | 3 ->
# 456 "asllib/Lexer.mll"
                               ( c_comments lexbuf                )
# 714 "asllib/Lexer.ml"

  | 4 ->
let
# 457 "asllib/Lexer.mll"
                 lxm
# 720 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 457 "asllib/Lexer.mll"
                               ( INT_LIT(Z.of_string lxm)         )
# 724 "asllib/Lexer.ml"

  | 5 ->
let
# 458 "asllib/Lexer.mll"
                 lxm
# 730 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 458 "asllib/Lexer.mll"
                               ( INT_LIT(Z.of_string lxm)         )
# 734 "asllib/Lexer.ml"

  | 6 ->
let
# 459 "asllib/Lexer.mll"
                  lxm
# 740 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 459 "asllib/Lexer.mll"
                               ( REAL_LIT(Q.of_string lxm)        )
# 744 "asllib/Lexer.ml"

  | 7 ->
# 460 "asllib/Lexer.mll"
                               ( string_lit (Buffer.create 16) lexbuf )
# 749 "asllib/Lexer.ml"

  | 8 ->
let
# 461 "asllib/Lexer.mll"
                    lxm
# 755 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) (lexbuf.Lexing.lex_curr_pos + -1) in
# 461 "asllib/Lexer.mll"
                               ( bitvector_lit lxm                )
# 759 "asllib/Lexer.ml"

  | 9 ->
let
# 462 "asllib/Lexer.mll"
                    lxm
# 765 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf (lexbuf.Lexing.lex_start_pos + 1) (lexbuf.Lexing.lex_curr_pos + -1) in
# 462 "asllib/Lexer.mll"
                               ( mask_lit lxm                     )
# 769 "asllib/Lexer.ml"

  | 10 ->
# 463 "asllib/Lexer.mll"
                               ( BNOT                             )
# 774 "asllib/Lexer.ml"

  | 11 ->
# 464 "asllib/Lexer.mll"
                               ( COMMA                            )
# 779 "asllib/Lexer.ml"

  | 12 ->
# 465 "asllib/Lexer.mll"
                               ( LT                               )
# 784 "asllib/Lexer.ml"

  | 13 ->
# 466 "asllib/Lexer.mll"
                               ( SHR                              )
# 789 "asllib/Lexer.ml"

  | 14 ->
# 467 "asllib/Lexer.mll"
                               ( BAND                             )
# 794 "asllib/Lexer.ml"

  | 15 ->
# 468 "asllib/Lexer.mll"
                               ( IMPL                             )
# 799 "asllib/Lexer.ml"

  | 16 ->
# 469 "asllib/Lexer.mll"
                               ( SHL                              )
# 804 "asllib/Lexer.ml"

  | 17 ->
# 470 "asllib/Lexer.mll"
                               ( RBRACKET                         )
# 809 "asllib/Lexer.ml"

  | 18 ->
# 471 "asllib/Lexer.mll"
                               ( RRBRACKET                        )
# 814 "asllib/Lexer.ml"

  | 19 ->
# 472 "asllib/Lexer.mll"
                               ( RPAR                             )
# 819 "asllib/Lexer.ml"

  | 20 ->
# 473 "asllib/Lexer.mll"
                               ( SLICING                          )
# 824 "asllib/Lexer.ml"

  | 21 ->
# 474 "asllib/Lexer.mll"
                               ( EQ                               )
# 829 "asllib/Lexer.ml"

  | 22 ->
# 475 "asllib/Lexer.mll"
                               ( LBRACE                           )
# 834 "asllib/Lexer.ml"

  | 23 ->
# 476 "asllib/Lexer.mll"
                               ( NEQ                              )
# 839 "asllib/Lexer.ml"

  | 24 ->
# 477 "asllib/Lexer.mll"
                               ( MINUS                            )
# 844 "asllib/Lexer.ml"

  | 25 ->
# 478 "asllib/Lexer.mll"
                               ( BEQ                              )
# 849 "asllib/Lexer.ml"

  | 26 ->
# 479 "asllib/Lexer.mll"
                               ( LBRACKET                         )
# 854 "asllib/Lexer.ml"

  | 27 ->
# 480 "asllib/Lexer.mll"
                               ( LLBRACKET                        )
# 859 "asllib/Lexer.ml"

  | 28 ->
# 481 "asllib/Lexer.mll"
                               ( LPAR                             )
# 864 "asllib/Lexer.ml"

  | 29 ->
# 482 "asllib/Lexer.mll"
                               ( DOT                              )
# 869 "asllib/Lexer.ml"

  | 30 ->
# 483 "asllib/Lexer.mll"
                               ( LEQ                              )
# 874 "asllib/Lexer.ml"

  | 31 ->
# 484 "asllib/Lexer.mll"
                               ( POW                              )
# 879 "asllib/Lexer.ml"

  | 32 ->
# 485 "asllib/Lexer.mll"
                               ( MUL                              )
# 884 "asllib/Lexer.ml"

  | 33 ->
# 486 "asllib/Lexer.mll"
                               ( RDIV                             )
# 889 "asllib/Lexer.ml"

  | 34 ->
# 487 "asllib/Lexer.mll"
                               ( EQ_OP                            )
# 894 "asllib/Lexer.ml"

  | 35 ->
# 488 "asllib/Lexer.mll"
                               ( BOR                              )
# 899 "asllib/Lexer.ml"

  | 36 ->
# 489 "asllib/Lexer.mll"
                               ( PLUS                             )
# 904 "asllib/Lexer.ml"

  | 37 ->
# 490 "asllib/Lexer.mll"
                               ( COLON                            )
# 909 "asllib/Lexer.ml"

  | 38 ->
# 491 "asllib/Lexer.mll"
                               ( ARROW                            )
# 914 "asllib/Lexer.ml"

  | 39 ->
# 492 "asllib/Lexer.mll"
                               ( BIARROW                          )
# 919 "asllib/Lexer.ml"

  | 40 ->
# 493 "asllib/Lexer.mll"
                               ( RBRACE                           )
# 924 "asllib/Lexer.ml"

  | 41 ->
# 494 "asllib/Lexer.mll"
                               ( fatal lexbuf (ObsoleteSyntax "string concatenation with ++") )
# 929 "asllib/Lexer.ml"

  | 42 ->
# 495 "asllib/Lexer.mll"
                               ( COLON_COLON                      )
# 934 "asllib/Lexer.ml"

  | 43 ->
# 496 "asllib/Lexer.mll"
                               ( GT                               )
# 939 "asllib/Lexer.ml"

  | 44 ->
# 497 "asllib/Lexer.mll"
                               ( PLUS_COLON                       )
# 944 "asllib/Lexer.ml"

  | 45 ->
# 498 "asllib/Lexer.mll"
                               ( STAR_COLON                       )
# 949 "asllib/Lexer.ml"

  | 46 ->
# 499 "asllib/Lexer.mll"
                               ( SEMI_COLON                       )
# 954 "asllib/Lexer.ml"

  | 47 ->
# 500 "asllib/Lexer.mll"
                               ( GEQ                              )
# 959 "asllib/Lexer.ml"

  | 48 ->
# 501 "asllib/Lexer.mll"
                               ( fatal lexbuf (ObsoleteSyntax "Loop limits with @looplimit") )
# 964 "asllib/Lexer.ml"

  | 49 ->
let
# 502 "asllib/Lexer.mll"
                    lxm
# 970 "asllib/Lexer.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_start_pos lexbuf.Lexing.lex_curr_pos in
# 502 "asllib/Lexer.mll"
                               ( tr_name lxm                      )
# 974 "asllib/Lexer.ml"

  | 50 ->
# 503 "asllib/Lexer.mll"
                               ( EOF                              )
# 979 "asllib/Lexer.ml"

  | 51 ->
# 504 "asllib/Lexer.mll"
                               ( raise LexerError                 )
# 984 "asllib/Lexer.ml"

  | __ocaml_lex_state -> lexbuf.Lexing.refill_buff lexbuf;
      __ocaml_lex_token_rec lexbuf __ocaml_lex_state

;;

# 505 "asllib/Lexer.mll"
 
end

# 995 "asllib/Lexer.ml"
